import 'package:flutter/material.dart';

import 'models/models.dart';


class ChartPainter extends CustomPainter {
  final List<String> x;
  final List<double> y;
  final double min, max;
  final GraphDirection graphDirection;
  static double border = 10.0;
  ChartPainter(
      {required this.x,
      required this.y,
      required this.min,
      required this.max,
      required this.graphDirection ,
      required Color lineColor,
      required Color bgColor});

  final linePaint = Paint()
    ..strokeWidth = 1.5
    ..style = PaintingStyle.stroke
    ..color = Colors.green;
  final dotPaintFill = Paint()
    ..strokeWidth = 1.0
    ..style = PaintingStyle.fill
    ..color = Colors.blueGrey[100]!;
  final labelStyle = TextStyle(
    color: Colors.purple,
    fontSize: 8,
  );
  final xlabelStyle = TextStyle(
    color: Colors.purple,
    fontSize: 9,
    fontWeight: FontWeight.w500
  );

  @override
  void paint(Canvas canvas, Size size) {
    final _clipRect = Rect.fromLTWH(0, 0, size.width, size.height);
    

    canvas.clipRect(_clipRect);
    canvas.drawPaint(Paint()..color = Colors.grey[200]!); // To give Background
    final drawAbleHeight = size.height - border * 2.0;
    final drawAbleWidth = size.width - border * 2.0;
    final hd = drawAbleHeight / 5.0;
    final wd = drawAbleWidth / x.length.toDouble();

    final height = hd * 3.0;
    final width = drawAbleWidth;

    if (height <= 0.0 || width <= 0.0) return;
    if (max - min < 1.0e-6) return;

    final hr = height / (max - min); //height per unit  value
    final left = border;
    final top = border;

    final c = Offset(left + wd / 2.0, top + height / 2.0);

    // _drawOutline(canvas, c, wd, height);

    final points = _computePoints(c, wd, height, hr);
    final path = _computePath(points);
    final labels = _computeLabels();
    canvas.drawPath(path, linePaint);
    //draw points
    drawDataPoints(points, canvas, c, dotPaintFill);
    //draw Labels
    drawLabelMethod(canvas, labels, c, wd, points, top);
    //draw X Labels
    final c1 = Offset(c.dx, top + 4.3 * hd);
    drawXlabels(
      canvas,
      c1,
      wd,
    );
  }

//Draw Points
  void drawDataPoints(
      List<Offset> points, Canvas canvas, Offset c, Paint dotPaintFill) {
    for (var i = 0; i < points.length; i++) {
      final dp = points[i];
      // final label = labels[i];
      // drawTextCentered(canvas,c,label ,labelStyle ,wd);
      canvas.drawCircle(dp, 3, dotPaintFill);
      canvas.drawCircle(dp, 3, linePaint);
      // c = c+ Offset(wd,0);
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }

  final outlinePaint = Paint()
    ..strokeWidth = 1.0
    ..style = PaintingStyle.stroke
    ..color = Colors.blue;

  // void _drawOutline(Canvas canvas, Offset c, double wd, double height) {
  //   y.forEach((p) {
  //     final react = Rect.fromCenter(center: c, width: wd, height: height);
  //     canvas.drawRect(react, outlinePaint);
  //     c = c + Offset(wd, 0);
  //   });
  // }

  List<Offset> _computePoints(Offset c, double wd, double height, double hr) {
    List<Offset> _points = [];
    if (graphDirection == GraphDirection.downToUp) {
     
      y.forEach((yp) {
      
        final yy = (max-yp) * hr;
        // final dp = Offset(c.dx, c.dy + height / 2.0 - yy);
        final dp = Offset(c.dx, c.dy - height / 2.0 + yy);
        _points.add(dp);
        c = c + Offset(wd, 0);
      });
    } else {
    
      y.forEach((yp) {
        final yy = (yp - min) * hr;
        final dp = Offset(c.dx, c.dy - height / 2.0 + yy);
        _points.add(dp);
        c = c + Offset(wd, 0);
      });
    }
    return _points;
  }

  Path _computePath(List<Offset> points) {
    final path = Path();
    for (int i = 0; i < points.length; i++) {
      final p = points[i];
      if (i == 0) {
        path.moveTo(p.dx, p.dy);
      } else {
        path.lineTo(p.dx, p.dy);
      }
    }
    return path;
  }

  List<String> _computeLabels() {
    return y.map((e) => "${e.toStringAsFixed(1)}").toList();
  }

  TextPainter measureText(
      String s, TextStyle style, double maxwidth, TextAlign align) {
    final span = TextSpan(text: s, style: style);
    final tp = TextPainter(
        text: span, textAlign: align, textDirection: TextDirection.ltr);
    tp.layout(minWidth: 0, maxWidth: maxwidth);
    return tp;
  }

  Size drawTextCentered(Canvas canvas, Offset c, String label,
      TextStyle labelStyle, double maxwidth) {
    final tp = measureText(label, labelStyle, maxwidth, TextAlign.center);
    final offset = c + Offset(-tp.width / 2.0, -tp.height / 2.0);
    tp.paint(canvas, offset);
    return tp.size;
  }

  void drawLabelMethod(Canvas canvas, List<String> labels, Offset c, double wd,
      List<Offset> points, double top) {
    int i = 0;
    labels.forEach((l) {
      final dp = points[i];
      final dy = dp.dy - 15.0 < top ? 15.0 : -15.0;
      final ly = dp + Offset(0, dy);
      drawTextCentered(canvas, ly, l, labelStyle, wd);
      i++;
      // c += Offset(wd,0);
    });
  }

  void drawXlabels(Canvas canvas, Offset c, double maxwd) {
    x.forEach((x) {
      drawTextCentered(canvas, c, x, xlabelStyle, maxwd);
      c = c + Offset(maxwd, 0);
    });
  }
}
